/*
 * interrupt.c
 *
 *  Created on: 27.05.2015
 *  Author: Ferdinand Goss, Julius Imbery, Markus Steimle
 */



#include <system.h>
#include <altera_avalon_pio_regs.h>
#include <altera_avalon_uart_regs.h>
#include <io.h>
#include <sys/alt_irq.h>


unsigned int nirq_isr_context;
void nIRQ_Initial(void);
void nPIO_ISR(void* isr_context);
void nUARTPI_ISR(void* isr_context);

/****************************************************************************
  Function:
    void nIRQ_Initial(void)

  Description:
    This function initializes the Interrupt for the Push Buttons and for the UART3

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
	UART3 is used for the communication with the Raspberry PI
  ***************************************************************************/
void nIRQ_Initial(void)
{
    IOWR_ALTERA_AVALON_PIO_IRQ_MASK(KEY_BASE, 3); //
    IOWR_ALTERA_AVALON_PIO_EDGE_CAP(KEY_BASE, 3); //
    // register ISR function
    alt_ic_isr_register(
    		KEY_IRQ_INTERRUPT_CONTROLLER_ID,
    		KEY_IRQ,
	  nPIO_ISR,
      0x0,
      0x0);

	alt_ic_isr_register(
				NEW_ULTRASOUND3_IRQ_INTERRUPT_CONTROLLER_ID,
				NEW_ULTRASOUND3_IRQ,
		  nUARTPI_ISR,
		  0x0,
		  0x0);
}


/****************************************************************************
  Function:
    void nPIO_ISR(void* nirq_isr_context)

  Description:
    This function is called when an interrupt from the Push-Buttons occurs.
	This means when a button is pressed.

  Precondition:
    Call nIRQ_Initial() prior to use this function

  Parameters:
    void* nirq_isr_context 

  Returns:
    None

  Remarks:
    nIRQ_Initial() registers the Interrupt from the Push-Buttons
	At the moment not used but maybe it is a good thing if somebody needs this.
  ***************************************************************************/
void nPIO_ISR(void* nirq_isr_context)
{
    IOWR_ALTERA_AVALON_PIO_EDGE_CAP(KEY_BASE, 1);
}

/****************************************************************************
  Function:
    void nUARTPI_ISR(void* nirq_isr_context)

  Description:
    This function is called when an interrupt from the UART3 occurs.
	It receives the bytes which was sent by the Raspberry PI.
	The first byte represents an identifier which represents which data 
	will be sent next. The second byte represents the value of the data. 
	According to the identifier steering and speed in percentage will be set.

  Precondition:
    Call nIRQ_Initial() prior to use this function

  Parameters:
    void* nirq_isr_context 

  Returns:
    None

  Remarks:
	Data with identifier 201 represents the steering
	Data with identifier 202 represents the speed in percentage
	Data with identifier 203 represents stop (speed = 0 and steering = 0)
  ***************************************************************************/
void nUARTPI_ISR(void* nirq_isr_context)
{
	int sr;
	unsigned char identifier;
	unsigned char data;

	sr = IORD_ALTERA_AVALON_UART_STATUS(NEW_ULTRASOUND3_BASE);

	if(sr & ALTERA_AVALON_UART_STATUS_RRDY_MSK)
	{
		identifier = IORD_ALTERA_AVALON_UART_RXDATA(NEW_ULTRASOUND3_BASE);
		if (identifier >= 201 && identifier <= 203)
		{
			data = receiveFromBoard();
			switch (identifier)
			{
				case 201: setSteering(data-7); break;

				case 202: setSpeedPerc(data-100); break;

				case 203: setSteering(0); setSpeedPerc(0); break;

				default: break;
			}
		}
		IOWR_ALTERA_AVALON_UART_STATUS(NEW_ULTRASOUND3_BASE, 0);
	}
}
